# Generated by Django 3.1.5 on 2021-01-18 15:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("books", "0003_auto_20201015_1222"),
    ]

    operations = [
        migrations.AddField(
            model_name="book",
            name="pages",
            field=models.IntegerField(null=True),
        ),
    ]
